@@ -14,6 +14,7 @@ from graphblas.exceptions import (
     DimensionMismatch,
     EmptyObject,
     IndexOutOfBound,
+    InvalidObject,
     InvalidValue,
     OutputNotEmpty,
 )
@@ -608,6 +609,28 @@ def test_assign_scalar_with_mask():
     assert v.isequal(result)
 
 
+def test_assign_list():
+    v = Vector(int, 4)
+    v[[0, 1]] = [2, 3]
+    expected = Vector.from_values([0, 1], [2, 3], size=4)
+    assert v.isequal(expected)
+    v[::2] = np.arange(2)
+    expected = Vector.from_values([0, 1, 2], [0, 3, 1], size=4)
+    assert v.isequal(expected)
+    with pytest.raises(TypeError):
+        v[0] = [1]
+    with pytest.raises(TypeError):
+        v()[0] = [1]
+    with pytest.raises(ValueError, match="shape mismatch"):
+        v[[0, 1]] = [1, 2, 3]
+    with pytest.raises(ValueError, match="shape mismatch"):
+        v[[0, 1]] = [[1, 2]]
+    with pytest.raises(ValueError, match="shape mismatch"):
+        v[[0, 1]] = [[1], [2]]
+    with pytest.raises(TypeError):
+        v[[0, 1]] = [2, object()]
+
+
 def test_apply(v):
     result = Vector.from_values([1, 3, 4, 6], [-1, -1, -2, 0])
     w = v.apply(unary.ainv).new()
@@ -668,6 +691,8 @@ def test_apply_indexunary(v):
     assert w2.isequal(v2)
     assert w3.isequal(v2)
     assert w4.isequal(v2)
+    with pytest.raises(TypeError, match="left"):
+        v.apply(indexunary.valueeq, left=s2)
 
 
 def test_select(v):
@@ -684,10 +709,63 @@ def test_select(v):
     assert w4.isequal(result)
     assert w5.isequal(result)
     assert w6.isequal(result)
+    with pytest.raises(TypeError, match="thunk"):
+        v.select(select.valueeq, object())
 
 
+@autocompute
+def test_select_bools_and_masks(v):
+    # Select with boolean and masks
+    result = Vector.from_values([1, 3], [1, 1], size=7)
+    w7 = v.select((v == 1).new()).new()
+    assert w7.isequal(result)
+    w7b = v.select(v == 1).new()  # we rewrite!
+    assert w7b.isequal(result)
+    w7c = v.select(~(v != 1)).new()
+    assert w7c.isequal(result)
+    w8 = v.select(w7.S).new()
+    assert w8.isequal(result)
+    w7[4] = 0
+    w9 = v.select(w7.V).new()
+    assert w9.isequal(result)
+    with pytest.raises(TypeError, match="thunk"):
+        v.select(w7.V, 777)
+    with pytest.raises(TypeError, match="thunk"):
+        v.select(v == 1, 777)
+    with pytest.raises(TypeError):
+        v.select(v.outer(v).new().S)
+    # Make sure we use `replace`
+    w9 << 1
+    w9 << v.select(w7.V)
+    assert w9.isequal(result)
+    # and with masks
+    w9 << 1
+    w9[1] = 0
+    w9(w9.V) << v.select(w7.V)
+    result2 = Vector.from_values([1, 3], [0, 1], size=7)
+    assert w9.isequal(result2)
+    w9 << 1
+    w9[1] = 0
+    w9(w9.V, replace=True) << v.select(w7.V)
+    del result2[1]
+    assert w9.isequal(result2)
+    w9 << 1
+    w9[1] = 0
+    w9(w9.V, binary.plus) << v.select(w7.V)
+    w8 << 1
+    w8[1] = 0
+    w8(w8.V, binary.plus) << v.dup(mask=w7.V)
+    assert w8.isequal(w9)
+    w9 << 1
+    w9(binary.plus) << v.select(w7.V)
+    w8 << 1
+    w8(binary.plus) << v.dup(mask=w7.V)
+    assert w8.isequal(w9)
+
+
+@pytest.mark.slow
 def test_indexunary_udf(v):
-    def twox_minusthunk(x, row, col, thunk):
+    def twox_minusthunk(x, row, col, thunk):  # pragma: no cover
         return 2 * x - thunk
 
     indexunary.register_new("twox_minusthunk", twox_minusthunk)
@@ -695,12 +773,14 @@ def test_indexunary_udf(v):
     assert not hasattr(select, "twox_minusthunk")
     with pytest.raises(ValueError):
         select.register_anonymous(twox_minusthunk)
+    with pytest.raises(TypeError, match="must be a function"):
+        select.register_anonymous(object())
     expected = Vector.from_values([1, 3, 4, 6], [-2, -2, 0, -4], size=7)
     result = indexunary.twox_minusthunk(v, 4).new()
     assert result.isequal(expected)
     delattr(indexunary, "twox_minusthunk")
 
-    def ii(x, idx, _, thunk):
+    def ii(x, idx, _, thunk):  # pragma: no cover
         return idx // 2 >= thunk
 
     select.register_new("ii", ii)
@@ -725,6 +805,10 @@ def test_reduce(v):
     assert v.reduce(binary.plus).new() == 4
     with pytest.raises(TypeError, match="Expected type: Monoid"):
         v.reduce(binary.minus)
+    with pytest.raises(TypeError, match="to get the Monoid"):
+        v.reduce(semiring.plus_times)
+    with pytest.raises(TypeError, match="part of a Monoid for BOOL datatype"):
+        v.reduce(binary.plus[bool])
 
     # Test accum
     s(accum=binary.times) << v.reduce(monoid.plus)
@@ -1859,6 +1943,15 @@ def test_udt():
     assert result.isequal(v)
     for aggop in [agg.any_value, agg.first, agg.last, agg.count, agg.first_index, agg.last_index]:
         v.reduce(aggop).new()
+    v.clear()
+    v[[0, 1]] = [(2, 3), (4, 5)]
+    expected = Vector.from_values([0, 1], [(2, 3), (4, 5)], dtype=udt, size=v.size)
+    vv = Vector.ss.deserialize(v.ss.serialize(), unsafe=True)
+    assert v.isequal(vv, check_dtype=True)
+    vv = Vector.ss.deserialize(v.ss.serialize(), dtype=v.dtype)
+    assert v.isequal(vv, check_dtype=True)
+    with pytest.raises(ValueError):
+        Vector.ss.deserialize(v.ss.serialize())
 
     # arrays as dtypes!
     np_dtype = np.dtype("(3,)uint16")
@@ -1904,6 +1997,33 @@ def test_udt():
     # Just make sure these work
     for aggop in [agg.any_value, agg.first, agg.last, agg.count, agg.first_index, agg.last_index]:
         v.reduce(aggop).new()
+    v.clear()
+    v[[0, 1]] = [[2, 3, 4], [5, 6, 7]]
+    expected = Vector.from_values([0, 1], [[2, 3, 4], [5, 6, 7]], dtype=udt2, size=v.size)
+    assert v.isequal(expected)
+    vv = Vector.ss.deserialize(v.ss.serialize(), unsafe=True)
+    assert v.isequal(vv, check_dtype=True)
+
+    long_dtype = np.dtype([("x", np.bool_), ("y" * 1000, np.float64)], align=True)
+    with pytest.warns(UserWarning, match="too large"):
+        long_udt = dtypes.register_anonymous(long_dtype)
+    v = Vector(long_udt, size=3)
+    v[0] = 0
+    v[1] = (1, 5.6)
+    vv = Vector.ss.deserialize(v.ss.serialize(), dtype=long_udt)
+    assert v.isequal(vv, check_dtype=True)
+    with pytest.raises(Exception):
+        # The size of the UDT name is limited
+        Vector.ss.deserialize(v.ss.serialize(), unsafe=True)
+    # May be able to look up non-anonymous dtypes by name if their names are too long
+    named_long_dtype = np.dtype([("x", np.bool_), ("y" * 1000, np.float64)], align=False)
+    with pytest.warns(UserWarning, match="too large"):
+        named_long_udt = dtypes.register_new("LongUDT", named_long_dtype)
+    v = Vector(named_long_udt, size=3)
+    v[0] = 0
+    v[1] = (1, 5.6)
+    vv = Vector.ss.deserialize(v.ss.serialize())
+    assert v.isequal(vv, check_dtype=True)
 
 
 def test_infix_outer():
@@ -2083,17 +2203,17 @@ def test_to_values_sort():
 
 
 def test_lambda_udfs(v):
-    result = v.apply(lambda x: x + 1).new()
+    result = v.apply(lambda x: x + 1).new()  # pragma: no branch
     expected = binary.plus(v, 1).new()
     assert result.isequal(expected)
-    result = v.ewise_mult(v, lambda x, y: x + y).new()
+    result = v.ewise_mult(v, lambda x, y: x + y).new()  # pragma: no branch
     expected = binary.plus(v & v).new()
     assert result.isequal(expected)
-    result = v.ewise_add(v, lambda x, y: x + y).new()
+    result = v.ewise_add(v, lambda x, y: x + y).new()  # pragma: no branch
     assert result.isequal(expected)
     # Should we also allow lambdas for monoids with assumed identity of 0?
     # with pytest.raises(TypeError):
-    v.ewise_add(v, lambda x, y: x + y)  # now okay
+    v.ewise_add(v, lambda x, y: x + y)  # pragma: no branch
     with pytest.raises(TypeError):
         v.inner(v, lambda x, y: x + y)
 
@@ -2107,3 +2227,27 @@ def test_get(v):
     with pytest.raises(ValueError):
         # Not yet supported
         v.get([0, 1])
+
+
+def test_ss_serialize(v):
+    for compression, level, nthreads in itertools.product(
+        [None, "none", "default", "lz4", "lz4hc"], [None, 1, 5, 9], [None, -1, 1, 10]
+    ):
+        if level is not None and compression != "lz4hc":
+            with pytest.raises(TypeError, match="level argument"):
+                v.ss.serialize(compression, level, nthreads=nthreads)
+            continue
+        a = v.ss.serialize(compression, level, nthreads=nthreads)
+        w = Vector.ss.deserialize(a, nthreads=nthreads)
+        assert v.isequal(w, check_dtype=True)
+    b = a.tobytes()
+    w = Vector.ss.deserialize(b)
+    assert v.isequal(w, check_dtype=True)
+    with pytest.raises(ValueError, match="compression argument"):
+        v.ss.serialize("bad")
+    with pytest.raises(ValueError, match="level argument"):
+        v.ss.serialize("lz4hc", -1)
+    with pytest.raises(ValueError, match="level argument"):
+        v.ss.serialize("lz4hc", 0)
+    with pytest.raises(InvalidObject):
+        Vector.ss.deserialize(a[:-5])
